/**
* ossl_threads.c - Necessary routines for using
* OpenSSL in a multithreaded application
*
* Copyright (C) 2004-2005 Pantelis Frangoudis, Elias Efstathiou
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*
* In addition, as a special exception, the copyright holders
* give permission to link the code of this program with the
* OpenSSL toolkit (or with modified versions of OpenSSL that use the
* same license as OpenSSL, version 0.9.8a), and distribute linked
* combinations including the two.  You must obey the GNU General Public License
* in all respects for all of the code used other than OpenSSL.  If you modify
* this file, you may extend this exception to your version of the
* file, but you are not obligated to do so.  If you do not wish to
* do so, delete this exception statement from your version.
*/

#include "ossl_threads.h"

//########################################################################
//callbacks for openssl thread-safety funcs

//returns a thread's identifier
unsigned long get_thread_id(void) {
	unsigned long retval;
	retval = (unsigned long)pthread_self();
	return retval;
}

void locking_callback(int mode, int type, char *file, int line) {
	if (mode & CRYPTO_LOCK) {
		pthread_mutex_lock(&(ossl_mutices[type]));
		lock_count[type]++;
	}
	else {
		pthread_mutex_unlock(&(ossl_mutices[type]));
	}
}
//########################################################################

//initialization function (inits mutices, etc, sets callbacks)
void init_ossl_threads() {
	int i;
	ossl_mutices = (pthread_mutex_t*)malloc(CRYPTO_num_locks() * sizeof(pthread_mutex_t));
	memset(ossl_mutices, 0, sizeof(pthread_mutex_t));

	lock_count = (long*)malloc(CRYPTO_num_locks() * sizeof(long));

	for (i=0; i<CRYPTO_num_locks(); i++) {
		lock_count[i]=0;
		pthread_mutex_init( &(ossl_mutices[i]), NULL);
	}

	CRYPTO_set_id_callback( (unsigned long (*)())get_thread_id);
	CRYPTO_set_locking_callback((void (*)())locking_callback);
}

//cleanup func (destroy mutices, free mem)
void cleanup_ossl_threads() {
	int i;
	CRYPTO_set_locking_callback(NULL);
	for (i=0; i<CRYPTO_num_locks(); i++) {
		pthread_mutex_destroy(&(ossl_mutices[i]));
	}
	SAFE_FREE(ossl_mutices);
	SAFE_FREE(lock_count);
}
