/**
* crypt.h - Cryptographic operations module
*
* Copyright (C) 2004-2005 Pantelis Frangoudis, Elias Efstathiou
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*
* In addition, as a special exception, the copyright holders
* give permission to link the code of this program with the
* OpenSSL toolkit (or with modified versions of OpenSSL that use the
* same license as OpenSSL, version 0.9.8a), and distribute linked
* combinations including the two.  You must obey the GNU General Public License
* in all respects for all of the code used other than OpenSSL.  If you modify
* this file, you may extend this exception to your version of the
* file, but you are not obligated to do so.  If you do not wish to
* do so, delete this exception statement from your version.
*/

#ifndef _PWNC_CRYPT_H
#define _PWNC_CRYPT_H

#include "io.h"

#include <openssl/rsa.h>
#include <openssl/bio.h>
#include <openssl/pem.h>
#include <openssl/sha.h>

#ifdef _ECC_
	#include <openssl/ecdsa.h>
	#include <openssl/ec.h>
#endif /* _ECC_*/

#include <openssl/err.h>
#include <openssl/lhash.h>

unsigned char *endian_conv(unsigned char* bytes, int bytelen);

//creates - signs receipt
PWNC_RCPT* construct_receipt(PWNC_CERT *cert, PWNC_PUBKEY *provpk, unsigned long timestamp, unsigned long weight, struct pwnc_key *key, int public);

PWNC_KEYPAIR* generate_key_pair(unsigned short algorithm, unsigned short bits);
PWNC_CERT* create_member_cert(PWNC_KEYPAIR *team_pair, PWNC_PUBKEY *user_pub);

void *struct_to_ossl_key(struct pwnc_key *key, unsigned short public);

#ifdef _ECC_
ECDSA_SIG *struct_to_ossl_ecdsa_sig(PWNC_SIG *signature);
#endif /* _ECC_*/

PWNC_SIG* pwnc_sign_msg(struct pwnc_key* key, unsigned char *message, int mlen, int public);
int verify_pwnc_signature(PWNC_SIG *signature, struct pwnc_key *key, unsigned char *message, int mlen, int public);

unsigned char* serialize_receipt_info(PWNC_CERT *cert, PWNC_PUBKEY *provpk, unsigned long timestamp, unsigned long weight, int *info_len);
unsigned char* serialize_cert_info(PWNC_PUBKEY *team, PWNC_PUBKEY *user, int *len);

#endif /* _PWNC_CRYPT_H */
