/**
* io.h - Input/Output operations 
* (e.g. reading P2PWNC messages from sockets, reading/writing
* public keys from disk, etc)
*
* Copyright (C) 2004-2005 Pantelis Frangoudis, Elias Efstathiou
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef _PWNC_IO_H
#define _PWNC_IO_H

#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <stdlib.h>
#include <netdb.h>
#include <unistd.h>
#include <sys/stat.h>

#include <pthread.h>

#include "types.h"
#include "conversions.h"

unsigned char* read_message(int sockfd, int *mtype);

PWNC_CERT* read_cert_file(const char *path);
int write_cert_file(PWNC_CERT *cert, const char *path);

struct pwnc_key* read_key_file(const char *path);
int write_key_file(struct pwnc_key *key , const char *path);

int write_rcpt_file(PWNC_RCPT *rcpt, const char* fmode, const char* path, int format);
PWNC_RCPT* read_rcpt_file(FILE* fp, long offset);

int io_store_receipt(PWNC_RCPT *rcpt, FILE *fp, int offset); //used by the rdb engine

#ifdef _LOGGING_
	#define LOG_MESSAGE(stream, args... ) \
		if (stream) fprintf(stream, args);
#else
	#define LOG_MESSAGE(stream, args...)
#endif

#endif /* _PWNC_IO_H */

