/**
* proto.h - P2PWNC protocol module constants
* Copyright (C) 2004-2005 Pantelis Frangoudis, Elias Efstathiou
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef _PWNC_PROTO_H
#define _PWNC_PROTO_H

#ifndef PWNC_SERVER_PORT
#define PWNC_SERVER_PORT 9999
#endif

#ifndef PWNC_TCA_PORT
#define PWNC_TCA_PORT 3333
#endif

//algorithms
#define PWNC_ALGO_RSA 10
#define PWNC_ALGO_ECC 20

//encoding formats
#define PWNC_FORMAT_B64 10
#define PWNC_FORMAT_RAW 20
//#define PWNC_FORMAT...proprietary...

//byte array types
//used in b642pem
#define PWNC_TYPE_PUBKEY 10
#define PWNC_TYPE_PRIVKEY 20
#define PWNC_TYPE_UCERT 30
#define PWNC_TYPE_RCPT 40

#define PWNC_ACTION_GRANT 10
#define PWNC_ACTION_FORBID 20

//pwnc protocol message types
#define PWNC_MSG_RCPT 10
#define PWNC_MSG_CONN 20
#define PWNC_MSG_RREQ 30
#define PWNC_MSG_CACK 40
#define PWNC_MSG_QUER 50
#define PWNC_MSG_QRSP 60
#define PWNC_MSG_RRSP 70
#define PWNC_MSG_TEAR 80
#define PWNC_MSG_UPDT 90

//some errors...
#define PWNC_ERR_NOT_VERIFIED 200
#define PWNC_ERR_RECEIPT_EXISTS 210 //receipt already exists
#define PWNC_ERR_RECEIPT_VERIFY 220 //receipt verification failed
#define PWNC_ERR_TEAM_LOOKUP 230 //at least one of the teams in a receipt does not exist
#define PWNC_ERR_OTHER 240 //other error
#define PWNC_ERR_CERT 250 //cetificate verification error

#define MODE_RCPT_REQ_ON 10
#define MODE_RCPT_REQ_OFF 20

#endif /* _PWNC_PROTO_H */
