/**
* threads.h - Thread pool implementation based on the POSIX threads API
*
* Copyright (C) 2004-2005 Pantelis Frangoudis, Elias Efstathiou
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef _PWNC_THREADS_H
#define _PWNC_THREADS_H

#include <pthread.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include <time.h>

typedef struct _client_thread_t {
	pthread_t client_thread; //client thread
	int is_free;
} CTHREAD;

typedef struct _client_thread_info_t {
	int client_socket;
	int thread_index;
} THREAD_INFO;

static pthread_mutex_t 	thread_pool_mutex;

//default pool size: 50 threads
int pool_size = 50;

CTHREAD * thread_pool = NULL;

void init_thread_pool(int max_threads);
int find_first_free_thread();
void  start_new_thread(void *thread_func, int sockfd);
void exit_client_thread(int thread_index);
void cancel_thread(int thread_index);
void thread_pool_cleanup();
int block_signals();

#endif /* _PWNC_THREADS_H */
