/**
* types.h - Data types representing P2PWNC entities
*
* Copyright (C) 2004-2005 Pantelis Frangoudis, Elias Efstathiou
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "types.h"

#include <string.h>
#include <stdlib.h>

struct pwnc_key *PWNC_KEY_new() {
	struct pwnc_key *key = NULL;
	ALLOC_AND_CLEAN(key, sizeof(struct pwnc_key), struct pwnc_key*);
	key->algorithm = 0;
	key->bits = 0;
	key->datalen=0;
	return key;
}
PWNC_CERT *PWNC_CERT_new() {
	PWNC_CERT *cert = NULL;
	ALLOC_AND_CLEAN(cert, sizeof(PWNC_CERT), PWNC_CERT*);
	cert->TeamPubKey = PWNC_KEY_new();
	cert->UserPubKey = PWNC_KEY_new();
	cert->TeamSig = PWNC_KEY_new();
	return cert;
}

PWNC_KEYPAIR *PWNC_KEYPAIR_new() {
	PWNC_KEYPAIR *pair = NULL;
	ALLOC_AND_CLEAN(pair, sizeof(PWNC_KEYPAIR), PWNC_KEYPAIR*);
	pair->pubkey = PWNC_KEY_new();
	pair->privkey = PWNC_KEY_new();
	return pair;
}

PWNC_RCPT *PWNC_RCPT_new() {
	PWNC_RCPT *rcpt = NULL;
	ALLOC_AND_CLEAN(rcpt, sizeof(PWNC_RCPT), PWNC_RCPT*);
	rcpt->ConsumerCert = PWNC_CERT_new();
	rcpt->ProviderPubKey = PWNC_KEY_new();
	rcpt->RcptSig = PWNC_KEY_new();
	rcpt->weight = 0;
	rcpt->timestamp = 0;
	return rcpt;
}
