/**
* fifo.c - FIFO queue data structure
*
* Copyright (C) 2004-2005 Pantelis Frangoudis, Elias Efstathiou
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "fifo.h"

//puts a node at the end of the Q (tail of the q is passed as a param)
void fifo_push(FIFO_NODE *to_be_enqueued, FIFO_NODE **fhead, FIFO_NODE **ftail) {
	//put the node at the end of the queue
	if ( (*fhead == NULL) && (*ftail == NULL) ) {//empty Q
		to_be_enqueued->next = NULL;
		to_be_enqueued->prev = NULL;
		*fhead = to_be_enqueued;
		*ftail = to_be_enqueued;
		return;
	}
	to_be_enqueued->next = NULL; //end of the Q
	to_be_enqueued->prev = *ftail;//new node's prev node is the current tail
	to_be_enqueued->prev->next = to_be_enqueued;
	*ftail = to_be_enqueued; //set tail to the new node
}

//pop (and return) the head of the queue
FIFO_NODE *fifo_pop(FIFO_NODE **fhead, FIFO_NODE **ftail) {
	FIFO_NODE *to_pop = *fhead;
	if (to_pop == NULL) {
		return NULL;
	}
	*fhead = (*fhead)->next;
	if (*fhead == NULL) {
		//popped the last element
		*ftail = NULL;
	}
	else {
		(*fhead)->prev = NULL;
	}
	return to_pop;
}

//traverse fifo using next pointers and pop each node
void fifo_empty(FIFO_NODE **fhead, FIFO_NODE **ftail) {
	FIFO_NODE *to_be_free = NULL;
	while (*fhead != NULL) {
		to_be_free = fifo_pop(fhead, ftail);
		if (to_be_free) {
			free(to_be_free);
			to_be_free = NULL;
		}
	}
}
